@@ -12,7 +12,7 @@
     you may not use this file except in compliance with the License.
     You may obtain a copy of the License at
 
-        http://www.apache.org/licenses/LICENSE-2.0
+        https://www.apache.org/licenses/LICENSE-2.0
 
     Unless required by applicable law or agreed to in writing, software
     distributed under the License is distributed on an "AS IS" BASIS,
@@ -136,7 +136,7 @@ class IsoDateTimeTzField(models.CharField):
             2015-11-11 22:21:37
             2015-11-11 22:21:37.123456
 
-        ... http://stackoverflow.com/questions/14368290/django-datetimefield-and-timezone-now
+        ... https://stackoverflow.com/questions/14368290/django-datetimefield-and-timezone-now
 
         ... so a direct comparison with ``DATETIME`` works only if the
         microsecond component is zero (or you coerce it to zero via
