@@ -39,6 +39,10 @@ import logging
 from typing import Any, Dict, Iterable, List, Optional, Sequence, Tuple, Union
 import unittest
 
+from cardinal_pythonlib.datetimefunc import (
+    coerce_to_pendulum,
+    pendulum_to_datetime_stripping_tz,
+)
 from cardinal_pythonlib.progress import ActivityCounter
 from cardinal_pythonlib.reprfunc import simple_repr
 try:
@@ -114,7 +118,7 @@ def check_attr_all_same(items: Sequence[Any],
             attribute from the first item to give some identifying context
             to the failure message.
         fail_if_different:
-            If true, raises ``ValueError`` on failure; otherwise, prints a
+            If true, raises :exc:`ValueError` on failure; otherwise, prints a
             warning to the log.
     """
     values = values_by_attr(items, attr)
@@ -459,7 +463,8 @@ class SheetHolder(object):
     def ensure_header(self, col: int,
                       header: Union[str, Sequence[str]]) -> None:
         """
-        Ensures that the header is correct for a specified column.
+        Ensures that the header is correct for a specified column, or raise
+        :exc:`ValueError`.
 
         You can specify a single correct heading or a sequence (e.g. list)
         of them.
@@ -469,8 +474,11 @@ class SheetHolder(object):
             return
         headers = self.headers
         if col < 0 or col >= len(headers):
+            max_col_idx = len(headers) - 1
             raise ValueError(
-                f"Bad column {col}; possible range is 0-{len(headers) - 1}"
+                f"Bad column index {col} ({column_lettering(col)}); "
+                f"possible range is 0-{max_col_idx} (columns "
+                f"{column_lettering(0)}-{column_lettering(max_col_idx)})"
             )
         v = headers[col]  # observed value
         self._checked_headers[col] = v  # cache for subsquent check
@@ -505,7 +513,11 @@ class SheetHolder(object):
         :meth:`ensure_header`).
         """
         if check_header is not None:
-            self.ensure_header(col, check_header)
+            try:
+                self.ensure_header(col, check_header)
+            except ValueError as e:
+                raise ValueError(
+                    f"When reading row with zero-based index {row}: {e}")
         v = self.sheet.cell_value(row, col)
         if v in self.null_values:
             return None
@@ -524,9 +536,19 @@ class SheetHolder(object):
         if none_or_blank_string(v):
             return default
         try:
-            return datetime.datetime(
-                *xlrd.xldate_as_tuple(v, self.book.datemode)
-            )
+            if isinstance(v, str):
+                # Sometimes we get strings, like "1800-01-01". These are
+                # outside the Excel range (1900 or 1904 onwards).
+                p_datetime = coerce_to_pendulum(v)
+                if p_datetime is None:
+                    raise ValueError
+                return pendulum_to_datetime_stripping_tz(p_datetime)
+            else:
+                # xlrd.xldate_as_tuple() converts an Excel number into a
+                # tuple: (year, month, day, hour, minute, nearest_second)
+                return datetime.datetime(
+                    *xlrd.xldate_as_tuple(v, self.book.datemode)
+                )
         except (TypeError, ValueError):
             raise ValueError(f"Bad date/time: {v!r}" + self._locinfo(row, col))
 
@@ -643,6 +665,34 @@ class SheetHolder(object):
             return default
         return str(v_int)
 
+    def read_str_nonfloat(self, row: int, col: int,
+                          default: str = None,
+                          check_header: Union[str, Sequence[str]] = None) \
+            -> Optional[str]:
+        """
+        Reads something that may be a string or numeric, but if it's numeric,
+        it's integer (not float). (This prevents e.g. "2" being read as a
+        floating-point value of "2.0" then converted to a string.)
+        """
+        v = self.read_value(row, col, check_header=check_header)
+        if none_or_blank_string(v):
+            return default
+        # See read_int() for more on the logic used here.
+        try:
+            fv = float(v)  # may raise
+        except (TypeError, ValueError):
+            # Not numeric
+            return v  # return the string version
+        # If we get here, v is numeric (e.g. "7" or "6.5" or 3.5).
+        try:
+            iv = int(v)
+        except (TypeError, ValueError):
+            # Numeric, but not integer
+            raise ValueError
+        if iv != fv:  # this picks up non-integer values
+            raise ValueError
+        return str(iv)  # string version of int
+
     def read_bool(self,
                   row: int,
                   col: int,
@@ -678,6 +728,19 @@ class SheetHolder(object):
         else:
             raise ValueError(f"Bad bool: {raw_v!r}" + self._locinfo(row, col))
 
+    def read_none(self, row: int, col: int,
+                  check_header: Union[str, Sequence[str]] = None) -> None:
+        """
+        Reads a value, and checks that it is a none/null value
+        """
+        v = self.read_value(row, col, check_header=check_header)
+        if v is not None:
+            raise ValueError(
+                f"Value should be none/null but is {v!r}" +
+                self._locinfo(row, col)
+            )
+        return None
+
     # -------------------------------------------------------------------------
     # Row generators
     # -------------------------------------------------------------------------
@@ -769,7 +832,7 @@ class RowHolder(object):
     def __init__(self, sheetholder: SheetHolder, row: int) -> None:
         self.sheetholder = sheetholder
         self.row = row  # zero-based index of our row
-        self._next_col = 0  # zero-based column index of the next column to read
+        self._next_col = 0  # zero-based column index of the next column to read  # noqa
 
     # -------------------------------------------------------------------------
     # Information
@@ -799,6 +862,26 @@ class RowHolder(object):
     def __repr__(self) -> str:
         return simple_repr(self, ["sheetholder", "row"])
 
+    # -------------------------------------------------------------------------
+    # Checks
+    # -------------------------------------------------------------------------
+    # More commonly, this is done via the "read...()" functions.
+
+    def ensure_header(self, col: int,
+                      header: Union[str, Sequence[str]]) -> None:
+        """
+        Ensures the column has an appropriate heading value, or raises
+        :exc:`ValueError`.
+        """
+        self.sheetholder.ensure_header(col, header)
+
+    def ensure_heading(self, col: int,
+                       header: Union[str, Sequence[str]]) -> None:
+        """
+        Synonym for :meth:`ensure_header`.
+        """
+        self.ensure_header(col, header)
+
     # -------------------------------------------------------------------------
     # Read operations, given a column number
     # -------------------------------------------------------------------------
@@ -868,6 +951,13 @@ class RowHolder(object):
         return self.sheetholder.read_str_int(
             self.row, col, default, check_header=check_header)
 
+    def read_str_nonfloat(self, col: int,
+                          default: str = None,
+                          check_header: Union[str, Sequence[str]] = None) \
+            -> Optional[str]:
+        return self.sheetholder.read_str_nonfloat(
+            self.row, col, default, check_header=check_header)
+
     def read_bool(self,
                   col: int,
                   default: bool = None,
@@ -886,6 +976,34 @@ class RowHolder(object):
             check_header=check_header
         )
 
+    def read_none(self,
+                  col: int,
+                  check_header: Union[str, Sequence[str]] = None) -> None:
+        return self.sheetholder.read_none(
+            row=self.row,
+            col=col,
+            check_header=check_header
+        )
+
+    # -------------------------------------------------------------------------
+    # Check the next column
+    # -------------------------------------------------------------------------
+
+    def ensure_next_col_header(self,
+                               header: Union[str, Sequence[str]]) -> None:
+        """
+        Ensures the next column has an appropriate heading value, or raises
+        :exc:`ValueError`.
+        """
+        self.ensure_header(self._next_col, header)
+
+    def ensure_next_col_heading(self,
+                                header: Union[str, Sequence[str]]) -> None:
+        """
+        Synonym for :meth:`ensure_next_col_header`.
+        """
+        self.ensure_next_col_header(header)
+
     # -------------------------------------------------------------------------
     # Read operations, incrementing the next column number automatically.
     # -------------------------------------------------------------------------
@@ -1011,6 +1129,20 @@ class RowHolder(object):
         self.inc_next_col()
         return v
 
+    def str_nonfloat_pp(self,
+                        default: str = None,
+                        check_header: Union[str, Sequence[str]] = None) \
+            -> Optional[str]:
+        """
+        Reads something that may be a string or numeric, but if it's numeric,
+        it's integer (not float). Then increments the "current" column.
+        Optionally, checks that the header for this column is as expected.
+        """
+        v = self.read_str_nonfloat(
+            self._next_col, default, check_header=check_header)
+        self.inc_next_col()
+        return v
+
     def bool_pp(
             self,
             default: bool = None,
@@ -1033,6 +1165,16 @@ class RowHolder(object):
         self.inc_next_col()
         return v
 
+    def none_pp(self, check_header: Union[str, Sequence[str]] = None) -> None:
+        """
+        Reads a null value, and ensures that it is null; then increments the
+        "current" column. Optionally, checks that the header for this column is
+        as expected.
+        """
+        self.read_none(col=self._next_col, check_header=check_header)
+        self.inc_next_col()
+        return None
+
 
 # =============================================================================
 # Self-testing
